/////////////////////////////////////////////////////////////////////////////
// CardDraw.c - Source file for card drawing routines
//
// Hearts Card Game
// Copyright (C) 2001 Leo C. Singleton IV
// http://www.leosingleton.com/projects/hearts/
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; version 2 of the
// License. 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Revision History
//
// 8/8/01 LCS - changed scrCopy to winPaint, scrCopyNOT to winInvert to
//		upgrade from Palm OS SDK 3.1 to SDK 4.0; Added extra set of bitmaps
//		for red numbers
/////////////////////////////////////////////////////////////////////////////


#include "CardDraw.h"


// local variables
static MemHandle BitmapH[17];
static MemHandle BitmapIH[17];
static WinHandle Bitmap[17];
static WinHandle BitmapI[17];

#ifdef COLOR
static MemHandle BitmapRH[13];
static MemHandle BitmapIRH[13];
static WinHandle BitmapR[13];
static WinHandle BitmapIR[13];
#endif // COLOR


extern void CheckLocks(void)
{
	UInt8 i;
	for (i=0;i<17;i++)
		if (MemHandleLockCount(BitmapH[i]) != 0)
			WinDrawChars("BitmapH Overlocked", 18, 10, 50);
	for (i=0;i<17;i++)
		if (MemHandleLockCount(BitmapIH[i]) != 0)
			WinDrawChars("BitmapIH Overlocked", 18, 10, 50);
}



/////////////////////////////////////////////////////////////////////////////
// PreDraw()
// Called before other drawing routines to load bitmaps
extern Boolean PreDraw(void)
{
	BitmapType *pBitmap;
	UInt16 err;
	UInt8 i;
	WinHandle window;

	window = WinGetDrawWindow();

	// load bitmaps into offscreen windows
	for(i=0; i<17; i++)
	{
		Bitmap[i] = (WinHandle)WinCreateOffscreenWindow(5, 7, screenFormat,
			&err);
		if (err)
			return false;

		BitmapH[i] = (MemHandle)DmGetResource(bitmapRsc, CardBegin + i);
		if (!BitmapH[i])
			return false;

		pBitmap = (BitmapType*)MemHandleLock(BitmapH[i]);
		WinSetDrawWindow(Bitmap[i]);
		WinDrawBitmap(pBitmap, 0, 0);
		MemHandleUnlock(BitmapH[i]);


		BitmapI[i] = (WinHandle)WinCreateOffscreenWindow(5, 7, screenFormat,
			&err);
		if (err)
			return false;

		BitmapIH[i] = (MemHandle)DmGetResource(bitmapRsc, CardIBegin + i);
		if (!BitmapIH[i])
			return false;

		pBitmap = (BitmapType*)MemHandleLock(BitmapIH[i]);
		WinSetDrawWindow(BitmapI[i]);
		WinDrawBitmap(pBitmap, 0, 0);
		MemHandleUnlock(BitmapIH[i]);
	}

#ifdef COLOR
	for(i=0; i<13; i++)
	{
		BitmapR[i] = (WinHandle)WinCreateOffscreenWindow(5, 7, screenFormat,
			&err);
		if (err)
			return false;

		BitmapRH[i] = (MemHandle)DmGetResource(bitmapRsc, CardRBegin + i);
		if (!BitmapRH[i])
			return false;

		pBitmap = (BitmapType*)MemHandleLock(BitmapRH[i]);
		WinSetDrawWindow(BitmapR[i]);
		WinDrawBitmap(pBitmap, 0, 0);
		MemHandleUnlock(BitmapRH[i]);


		BitmapIR[i] = (WinHandle)WinCreateOffscreenWindow(5, 7, screenFormat,
			&err);
		if (err)
			return false;

		BitmapIRH[i] = (MemHandle)DmGetResource(bitmapRsc, CardIRBegin + i);
		if (!BitmapIRH[i])
			return false;

		pBitmap = (BitmapType*)MemHandleLock(BitmapIRH[i]);
		WinSetDrawWindow(BitmapIR[i]);
		WinDrawBitmap(pBitmap, 0, 0);
		MemHandleUnlock(BitmapIRH[i]);
	}
#endif // COLOR

// we don't use card backs in hearts, so this part is commented out
/*	for (i=0; i<CardBacks; i++)
	{
		BitmapB[i] = (WinHandle)WinCreateOffscreenWindow(18, 28,
			screenFormat, &err);
		if (err)
			return false;

		BitmapBH[i] = (MemHandle)DmGetResource(bitmapRsc, CardBackBegin + i);
		if (!BitmapBH[i])
			return false;

		pBitmap = (BitmapType*)MemHandleLock(BitmapBH[i]);
		WinSetDrawWindow(BitmapB[i]);
		WinDrawBitmap(pBitmap, 0, 0);
		MemHandleUnlock(BitmapBH[i]);
	}
*/
	WinSetDrawWindow(window);

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// DrawCard()
// Draws the specified card at the specified x-y location
//extern Boolean DrawCard(UInt8 card, UInt8 suit, UInt8 cardType,
//						Boolean invert, UInt16 x, UInt16 y)
extern Boolean DrawCard(const UInt8 card, const UInt8 suit,
						const UInt8 cardType, const Boolean invert,
						const UInt16 x, const UInt16 y)
{
	RectangleType srcRect, destRect;
	
	srcRect.topLeft.x = 0;
	srcRect.topLeft.y = 0;
	srcRect.extent.x = 5;
	srcRect.extent.y = 7;
	destRect.topLeft.x = x + 1;
	destRect.topLeft.y = y + 1;

	// draw frame
	switch (cardType)
	{
	case CARDTYPE_FULL:
		WinDrawLine(x + 1, y, x + 18, y);
		WinDrawLine(x + 1, y + 29, x + 18, y + 29);
		WinDrawLine(x, y + 1, x, y + 28);
		WinDrawLine(x + 19, y + 1, x + 19, y + 28);
		destRect.extent.x = 18;
		destRect.extent.y = 28;
		break;

	case CARDTYPE_TOP:
		WinDrawLine(x + 1, y, x + 18, y);
		WinDrawLine(x, y + 1, x, y + 9);
		WinDrawLine(x + 19, y + 1, x + 19, y + 9);
		destRect.extent.x = 18;
		destRect.extent.y = 9;
		break;

	case CARDTYPE_LEFT:
		WinDrawLine(x + 1, y, x + 9, y);
		WinDrawLine(x + 1, y + 29, x + 9, y + 29);
		WinDrawLine(x, y + 1, x, y + 28);
		destRect.extent.x = 9;
		destRect.extent.y = 28;
		break;
	}

	// clear card area
	WinEraseRectangle(&destRect, 0);

	// draw bitmaps
#ifdef COLOR
	if (suit == SUIT_DIAMONDS || suit == SUIT_HEARTS)	// this is weird - it appears to work backwards - there must be an incorrect #define somewhere
		WinCopyRectangle(BitmapR[card], 0, &srcRect, x + 2, y + 2, winPaint);
	else
#endif
		WinCopyRectangle(Bitmap[card], 0, &srcRect, x + 2, y + 2, winPaint);
	
	if (cardType != CARDTYPE_TOP)
		WinCopyRectangle(Bitmap[suit + 13], 0, &srcRect, x + 2, y + 10,
			winPaint);
	else
		WinCopyRectangle(Bitmap[suit + 13], 0, &srcRect, x + 13, y + 2,
			winPaint);

	// draw flipped bitmaps
	if (cardType == CARDTYPE_FULL)
	{
		WinCopyRectangle(BitmapI[suit + 13], 0, &srcRect, x + 13, y + 13,
			winPaint);
#ifdef COLOR
		if (suit == SUIT_DIAMONDS || suit == SUIT_HEARTS)
			WinCopyRectangle(BitmapIR[card], 0, &srcRect, x + 13, y + 21,
				winPaint);
		else
#endif
			WinCopyRectangle(BitmapI[card], 0, &srcRect, x + 13, y + 21,
				winPaint);
	}

	// invert card, if necessary
	if (invert)
		WinInvertRectangle(&destRect, 0);

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// DrawBack()
// Draws the specified card back at the specified x-y location
/*
extern Boolean DrawBack(UInt8 back, UInt8 cardType, Boolean invert,
						UInt16 x, UInt16 y)
{
	RectangleType srcRect;

	srcRect.topLeft.x = 0;
	srcRect.topLeft.y = 0;

	// draw frame
	switch (cardType)
	{
	case CARDTYPE_FULL:
		WinDrawLine(x + 1, y, x + 18, y);
		WinDrawLine(x + 1, y + 29, x + 18, y + 29);
		WinDrawLine(x, y + 1, x, y + 28);
		WinDrawLine(x + 19, y + 1, x + 19, y + 28);
		srcRect.extent.x = 18;
		srcRect.extent.y = 28;
		break;

	case CARDTYPE_TOP:
		WinDrawLine(x + 1, y, x + 18, y);
		WinDrawLine(x, y + 1, x, y + 9);
		WinDrawLine(x + 19, y + 1, x + 19, y + 9);
		srcRect.extent.x = 18;
		srcRect.extent.y = 9;
		break;

	case CARDTYPE_LEFT:
		WinDrawLine(x + 1, y, x + 9, y);
		WinDrawLine(x + 1, y + 29, x + 9, y + 29);
		WinDrawLine(x, y + 1, x, y + 28);
		srcRect.extent.x = 9;
		srcRect.extent.y = 28;
		break;

	case CARDTYPE_TOPSMALL:
		WinDrawLine(x + 1, y, x + 18, y);
		WinDrawLine(x, y + 1, x, y + 2);
		WinDrawLine(x + 19, y + 1, x + 19, y + 2);
		srcRect.extent.x = 18;
		srcRect.extent.y = 1;
		break;

	case CARDTYPE_LEFTSMALL:
		WinDrawLine(x + 1, y, x + 2, y);
		WinDrawLine(x + 1, y + 29, x + 2, y + 29);
		WinDrawLine(x, y + 1, x, y + 28);
		srcRect.extent.x = 1;
		srcRect.extent.y = 28;
		break;
	}

	// copy bitmap
	WinCopyRectangle(BitmapB[back], 0, &srcRect, x + 1, y + 1,
		invert ? winInvert : winPaint);

	return true;
}
*/

/////////////////////////////////////////////////////////////////////////////
// PostDraw()
// Called after other drawing routines to close bitmaps
extern Boolean PostDraw(void)
{
	UInt8 i;
	
	// release bitmaps
	for(i=0; i<17; i++)
	{
		if (Bitmap[i])
			WinDeleteWindow(Bitmap[i], false);
		DmReleaseResource(BitmapH[i]);

		if (BitmapI[i])
			WinDeleteWindow(BitmapI[i], false);
		DmReleaseResource(BitmapIH[i]);
	}

#ifdef COLOR
	for (i=0; i<13; i++)
	{
		if (BitmapR[i])
			WinDeleteWindow(BitmapR[i], false);
		DmReleaseResource(BitmapRH[i]);

		if (BitmapIR[i])
			WinDeleteWindow(BitmapIR[i], false);
		DmReleaseResource(BitmapIRH[i]);
	}
#endif // COLOR

/*	for(i=0; i<CardBacks; i++)
	{
		if (BitmapB[i])
			WinDeleteWindow(BitmapB[i], false);
		DmReleaseResource(BitmapBH[i]);
	}*/

	return true;
}

